@@ -23,9 +23,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLExtendedFetch.c,v 1.7 2004/07/24 17:55:38 lurcher Exp $
+ * $Id: SQLExtendedFetch.c,v 1.9 2005/10/27 17:54:49 lurcher Exp $
  *
  * $Log: SQLExtendedFetch.c,v $
+ * Revision 1.9  2005/10/27 17:54:49  lurcher
+ * fix what I suspect is a typo in qt.m4
+ *
+ * Revision 1.8  2005/10/21 16:49:53  lurcher
+ * Fix a problem with the cursor lib and rowsets
+ *
  * Revision 1.7  2004/07/24 17:55:38  lurcher
  * Sync up CVS
  *
@@ -83,6 +89,8 @@
 
 #define ABS(x)  (((x)>=0)?(x):(-(x)))
 
+#define SQL_FETCH_PART_ROWSET SQL_NO_DATA + 1
+
 SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
 {
     SQLSMALLINT ret;
@@ -228,7 +236,6 @@ SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
         ret = SQLFETCH( cl_statement -> cl_connection,
                         cl_statement -> driver_stmt );
 
-
         if ( ret == SQL_NO_DATA )
         {
             /*
@@ -472,6 +479,17 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
         }
         ret = SQL_SUCCESS;
     }
+
+    if ( ret == SQL_NO_DATA && row > 0 )
+	{
+        *fetched_rows = row;
+    	if ( rows_fetched_ptr )
+		{
+        	*rows_fetched_ptr = row_count;
+		}
+		ret = SQL_FETCH_PART_ROWSET;
+	}
+
     if ( SQL_SUCCEEDED( ret ))
     {
         *fetched_rows = row;
@@ -489,7 +507,8 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         int fetch_orientation,
         int fetch_offset,
         SQLUSMALLINT *row_status_ptr,
-        SQLUINTEGER *rows_fetched_ptr )
+        SQLUINTEGER *rows_fetched_ptr,
+		int ext_fetch )
 {
     SQLRETURN ret;
     int rows_in_set, row_offset, fetched_rows, info = 0;
@@ -515,10 +534,20 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         cl_statement -> first_fetch_done = 1;
     }
 
-    if ( cl_statement -> rowset_array_size < 1 )
-        rows_in_set = 1;
-    else 
-        rows_in_set = cl_statement -> rowset_array_size;
+	if ( ext_fetch ) 
+	{
+    	if ( cl_statement -> rowset_size < 1 )
+       	 	rows_in_set = 1;
+    	else 
+        	rows_in_set = cl_statement -> rowset_size;
+	}
+	else 
+	{
+    	if ( cl_statement -> rowset_array_size < 1 )
+       	 	rows_in_set = 1;
+    	else 
+        	rows_in_set = cl_statement -> rowset_array_size;
+	}
 
     /*
      * I refuse to document all these conditions, you have to
@@ -539,6 +568,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_NEXT:
@@ -568,6 +601,11 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+        	cl_statement -> rowset_position == CL_AFTER_END;
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_PRIOR:
@@ -623,6 +661,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         break;
 
       case SQL_FETCH_RELATIVE:
@@ -635,7 +677,8 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                     SQL_FETCH_ABSOLUTE,
                     fetch_offset,
                     row_status_ptr,
-                    rows_fetched_ptr );
+                    rows_fetched_ptr, 
+					ext_fetch );
         }
         else if ( cl_statement -> rowset_position == CL_BEFORE_START &&
             fetch_offset <= 0 )
@@ -712,6 +755,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
 
         break;
 
@@ -804,6 +851,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
                 cl_statement -> rowset_position;
             cl_statement -> rowset_position += fetched_rows;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
 
         break;
 
@@ -840,6 +891,10 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
             cl_statement -> curr_rowset_start = 
                 cl_statement -> rowset_position = CL_AFTER_END;
         }
+		else if ( ret == SQL_FETCH_PART_ROWSET ) 
+		{
+			ret = SQL_SUCCESS;
+		}
         
         break;
     }
@@ -873,5 +928,6 @@ SQLRETURN CLExtendedFetch(
             f_fetch_type, 
             irow,
             rgf_row_status,
-            pcrow );
+            pcrow,
+			1 );
 }
