@@ -13,6 +13,7 @@
 
 #define UNICODE
 #include "isql.h"
+#include "ini.h"
 #include "sqlucode.h"
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
@@ -249,10 +250,11 @@ int main( int argc, char *argv[] )
  ***************************/
 int OpenDatabase( SQLHENV *phEnv, SQLHDBC *phDbc, char *szDSN, char *szUID, char *szPWD )
 {
-    SQLTCHAR dsn[ 1024 ], uid[ 1024 ], pwd[ 1024 ];
+    SQLCHAR dsn[ 1024 ], uid[ 1024 ], pwd[ 1024 ];
     SQLTCHAR cstr[ 1024 ];
-    char zcstr[ 1024 ];
+    char zcstr[ 1024 ], tmp[ 1024 ];
     int i;
+	size_t zclen;
 
 	if ( SQLAllocEnv( phEnv ) != SQL_SUCCESS )
 	{
@@ -268,23 +270,62 @@ int OpenDatabase( SQLHENV *phEnv, SQLHDBC *phDbc, char *szDSN, char *szUID, char
 		return 0;
 	}
 
-    for ( i = 0; i < strlen( szDSN ); i ++ )
-    {
-        dsn[ i ] = szDSN[ i ];
-    }
+	if ( szDSN ) 
+	{
+		size_t DSNlen=strlen( szDSN );
+    	for ( i = 0; i < DSNlen; i ++ )
+    	{
+        	dsn[ i ] = szDSN[ i ];
+    	}
+		dsn[ i ] = '\0';
+	}
+	else 
+	{
+		dsn[ 0 ] = '\0';
+	}
 
-    for ( i = 0; i < strlen( szUID ); i ++ )
-    {
-        uid[ i ] = szUID[ i ];
-    }
+	if ( szUID )
+	{
+		size_t UIDlen=strlen( szUID );
+    	for ( i = 0; i < UIDlen; i ++ )
+    	{
+        	uid[ i ] = szUID[ i ];
+    	}
+		uid[ i ] = '\0';
+	}
+	else 
+	{
+		uid[ 0 ] = '\0';
+	}
 
-    for ( i = 0; i < strlen( szPWD ); i ++ )
-    {
-        pwd[ i ] = szPWD[ i ];
-    }
+	if ( szPWD ) 
+	{
+		size_t PWDlen=strlen( szPWD );
+    	for ( i = 0; i < PWDlen; i ++ )
+    	{
+        	pwd[ i ] = szPWD[ i ];
+    	}
+		pwd[ i ] = '\0';
+	}
+	else 
+	{
+		pwd[ 0 ] = '\0';
+	}
 
-    sprintf( zcstr, "DSN=%s;UID=%s;PWD=%s", szDSN, szUID, szPWD );
-    for ( i = 0; i < strlen( zcstr ); i ++ )
+    sprintf( zcstr, "DSN=%s", dsn );
+	if ( uid ) 
+	{
+    	sprintf( tmp, ";UID=%s", uid );
+		strcat( zcstr, tmp );
+	}
+	if ( pwd ) 
+	{
+    	sprintf( tmp, ";PWD=%s", pwd );
+		strcat( zcstr, tmp );
+	}
+
+	zclen=strlen( zcstr );
+    for ( i = 0; i < zclen; i ++ )
     {
         cstr[ i ] = zcstr[ i ];
     }
@@ -365,7 +406,7 @@ int ExecuteSQL( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, in
         if ( SQLNumResultCols( hStmt, &cols ) != SQL_SUCCESS )
         {
             if ( bVerbose ) DumpODBCLog( hEnv, hDbc, hStmt );
-            fprintf( stderr, "[ISQL]ERROR: Could not SQLNunResultCols\n" );
+            fprintf( stderr, "[ISQL]ERROR: Could not SQLNumResultCols\n" );
             SQLFreeStmt( hStmt, SQL_DROP );
             return 0;
         }
@@ -753,11 +794,11 @@ SQLLEN WriteBodyNormal( SQLHSTMT hStmt )
                 if ( strlen((char*)szColumnValue) < max( nMaxLength, strlen((char*)szColumnName )))
                 {
                     int i;
+					size_t maxlen=max( nMaxLength, strlen((char*)szColumnName ));
                     strcpy((char*) szColumn, "| " );
                     strcat((char*) szColumn, (char*) szColumnValue );
 
-                    for ( i = strlen((char*) szColumnValue ); 
-                        i < max( nMaxLength, strlen((char*)szColumnName )); i++ )
+                    for ( i = strlen((char*) szColumnValue ); i < maxlen; i ++ )
                     {
                         strcat((char*) szColumn, " " );
                     }
